#version 330 core

in vec4 fPosition;
in vec3 fNormals;
in vec2 fTexCoords;

out vec4 color;

uniform float alpha;

layout (std140) uniform Material {
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
} material;

uniform sampler2D meshTexture;

vec3 getLightDirection(vec4 pos)
{
	vec4 lightPosition = vec4(100.0, 50.0, 100.0, 1.0);
	vec4 diff = lightPosition - pos;
	//diff = diff;
	return diff.xyz;
}

void main()
{
	vec3 lightDirection = getLightDirection(fPosition); 
	vec3 viewDirection = vec3(0.0, 0.0, 1.0);
	float dist = length(lightDirection);
	dist = 1.4;
	lightDirection = normalize(lightDirection);

	vec4 fragColor = vec4(0.0);
	vec4 ambient = vec4(0.0);
	if (material.texCount > 0) {
		fragColor = texture2D(meshTexture, fTexCoords.st);
		ambient = 0.3 * fragColor;
		if (fragColor.a == 0) {
			discard;
			return;
		}
	} else {
		fragColor = material.diffuse;
		ambient = material.ambient;
	}

	float diffuseTerm = clamp(dot(lightDirection, fNormals), 0.0, 1.0);
	vec3 h = normalize(lightDirection + viewDirection);
	float specularTerm = pow(clamp(dot(fNormals, h),0,1), material.shininess);
	specularTerm = 0;

	color = ambient + diffuseTerm * fragColor / dist + specularTerm * material.specular / dist;
	color.a = fragColor.a;
}
