#version 330 core

in vec4 fPosition;
in vec2 fTexCoords;
in vec3 eyeToFragment;
in vec4 nearPlane;
in vec4 spherePosition;

out vec4 color;

uniform int width;
uniform int height;

uniform sampler2D depthTexture;
uniform sampler2D colorTexture;
uniform sampler2D perlinTexture1;
uniform sampler2D perlinTexture2;
uniform sampler2D glowTexture;

uniform vec3 cameraPosition;

uniform float fogTime;


void main()
{
	float znear = 0.1;
	float zfar = 100.0;

	vec4 depth = texture2D(depthTexture, fTexCoords.st);
	float d = 2.0 * depth.x - 1.0; // [-1,1]
	float z = 2.0 * znear * zfar / (zfar + znear - d * (zfar - znear));

	vec3 fragmentPos = cameraPosition + z * normalize(eyeToFragment);

	float y = fragmentPos.y;
	float delta = y - cameraPosition.y;
	if (delta == 0) delta = 0.01;
	float n = texture2D(perlinTexture1, fTexCoords.st).r;
	float n2 = texture2D(perlinTexture2, fTexCoords.st).r;

	float dist = sqrt(dot(cameraPosition.xy, cameraPosition.xy));
	n = mix(n, n2, fogTime); //dist/64*3);
	float F = z/delta * (-exp(-y) + exp(-cameraPosition.y)) * 0.3;
	float f = exp(-F * n);
	f = clamp(f, 0, 1);

	vec4 fog = vec4(0.8, 0.8, 0.8, 1.0);
	//f = max(0.3, f);
	color = texture2D(colorTexture, fTexCoords.st) + texture2D(glowTexture, fTexCoords.st);
	color = color * f + fog * (1-f);
}
