#version 330 core

layout(location = 0) in vec3 vert;
layout(location = 1) in vec3 normals;
layout(location = 2) in vec2 texCoords;
layout(location = 3) in vec3 tangents;
layout(location = 4) in vec3 bitangents;

out vec4 fPosition;
out vec3 fNormals;
out vec2 fTexCoords;
out vec4 shadowPosition;
out vec3 fTangents;
out vec3 fBitangents;

out vec3 lightDirection_tan;
out vec3 eyeDirection_tan;

uniform mat4 M;
uniform mat4 Minv;
uniform mat4 V;
uniform mat4 Vinv;
uniform mat4 P;
uniform mat4 shadowMatrix;

uniform int shadowMapWidth;
uniform int shadowMapHeight;
uniform vec3 lightDirection;
uniform vec3 cameraPosition;

vec4 getLightDirection(vec4 pos)
{
	vec4 lightPosition = vec4(100.0, 50.0, -100.0, 1.0);
	vec4 diff = lightPosition - pos;
	//diff = diff;
	return diff;
}

void main()
{
	gl_Position = P * V * M * vec4(vert, 1.0);
	mat4 normalMatrix = transpose(Minv * Vinv);
	fNormals = normalize(vec3(normalMatrix * vec4(normals, 0.0)));
	fTexCoords = texCoords;
	fPosition = M * vec4(vert, 1.0);
	shadowPosition = shadowMatrix * M * vec4(vert, 1.0);

	vec3 tangent = normalize(tangents);
	vec3 bitangent = normalize(bitangents);
	vec3 normal = normalize(normals);

	mat3 TBN = transpose(mat3(tangent,bitangent,normal));

	vec3 lightDir = vec3(Minv * vec4(-lightDirection, 0.0));
	lightDirection_tan = normalize(TBN * lightDir);

	vec4 eyeDirWorld = vec4(cameraPosition, 1.0) - fPosition;
	vec3 eyeDir = vec3(Minv * eyeDirWorld);
	eyeDirection_tan = normalize(TBN * eyeDir);

	fTangents = tangents;
	fBitangents = bitangents;
}
