#version 330 core

in vec4 fPosition;
in vec3 fNormals;
in vec2 fTexCoords;
in vec4 shadowPosition;
in vec3 fLightDirection;
in vec3 fColor;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 glow;

uniform float alpha;

layout (std140) uniform Material {
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
} material;

uniform sampler2D meshTexture;
uniform sampler2D shadowMap;

uniform int useColor;

vec3 getLightDirection(vec4 pos)
{
	vec4 lightPosition = vec4(100.0, 50.0, -100.0, 1.0);
	vec4 diff = lightPosition - pos;
	//diff = diff;
	return diff.xyz;
}

void main()
{
	//vec3 lightDirection = getLightDirection(fPosition); 
	vec3 viewDirection = vec3(0.0, 0.0, 1.0);
	float dist = 1.4;
	vec3 lightDir = normalize(fLightDirection);
	
	if (useColor == 1) {
		color = vec4(fColor, 1.0); return;
	}

	vec4 fragColor = vec4(0.0);
	vec4 ambient = vec4(0.0);
	if (material.texCount > 0) {
		fragColor = texture2D(meshTexture, fTexCoords.st);
		ambient = 0.1 * fragColor;
	} else {
		fragColor = material.diffuse;
		ambient = material.ambient;
	}

	float diffuseTerm = clamp(dot(lightDir, fNormals), 0.0, 1.0);
	vec3 h = normalize(lightDir + viewDirection);
	float specularTerm = pow(clamp(dot(fNormals, h),0,1), material.shininess);
	specularTerm = 0;

	color = ambient + diffuseTerm * fragColor / dist + specularTerm * material.specular / dist;
	color.a = alpha;

	vec4 coord = shadowPosition;
	coord = coord/coord.w;
	if (any(lessThan(coord.st, vec2(0.0))) || any(greaterThan(coord.st, vec2(1.0)))) {
		return;
	}

	vec2 poissonDisk[4] = vec2[](
	  vec2( -0.94201624, -0.39906216 ),
	  vec2( 0.94558609, -0.76890725 ),
	  vec2( -0.094184101, -0.92938870 ),
	  vec2( 0.34495938, 0.29387760 )
	);

	float visibility = 1.0;
	float depth = shadowPosition.z / shadowPosition.w;
	for (int i = 0; i < 4; ++i) {
		float result = texture2D(shadowMap, vec2(shadowPosition.xy + poissonDisk[i] / 50) / shadowPosition.w).r;
		if (result < depth) {
			visibility -= 0.15;
		}
	}
	color.rgb *= clamp(visibility, 0.0, 1.0);
	glow = vec4(0.0, 0.0, 0.0, 1.0);
	if (material.emissive.r > 0) {
		glow = vec4(vec3(fragColor * diffuseTerm), 1.0);
	}
	/*
	if (texture2D(shadowMap, coord.st).r < coord.z) {
		color.rgb = color.rgb * 0.5;
	}
	*/
}
