#version 330 core

in vec4 fPosition;
in vec3 fNormals;
in vec2 fTexCoords;
in vec4 shadowPosition;
in vec3 fLightDirection;

layout(location = 0) out vec4 color;
layout(location = 1) out vec4 glow;

uniform sampler2D heightMap;
uniform sampler2D wayMap;
uniform sampler2D shadowMap;
uniform sampler2D grassTexture;
uniform sampler2D stoneTexture;
uniform sampler2D snowTexture;
uniform sampler2D wayTileTexture;

uniform int width;
uniform int height;

void main()
{
	float GRASS_H = 0.4;
	float ROCK_H = 1.2;

	vec3 viewDirection = vec3(0.0, 0.0, 1.0);
	float dist = 1.5;
	vec3 lightDir = normalize(fLightDirection);

	vec2 texC = (fPosition.xz + vec2(width/2, height/2) + vec2(0.5)) / vec2(width, height);
	float h = texture2D(heightMap, texC).r;
	float way = texture2D(wayMap, texC).r;

	vec4 grassColor = texture2D(grassTexture, fTexCoords.st);
	vec4 stoneColor = texture2D(stoneTexture, fTexCoords.st);
	vec4 snowColor = texture2D(snowTexture, fTexCoords.st);
	vec4 wayColor = texture2D(wayTileTexture, fTexCoords.st);
	
	float grassHeight = h / GRASS_H;
	float rockHeight = (h - GRASS_H) / (ROCK_H - GRASS_H);

	vec4 rockMix = mix(grassColor, stoneColor, clamp(grassHeight, 0, 1));
	vec4 terrainColor = mix(rockMix, snowColor, clamp(rockHeight, 0, 1));
	vec4 fragColor = mix(terrainColor, wayColor, way);
	
	vec4 ambient = 0.15 * fragColor;

	float diffuseTerm = clamp(dot(lightDir, fNormals), 0.0, 1.0);
	
	color = ambient + diffuseTerm * fragColor / dist;
	color.a = 1.0;

	vec4 coord = shadowPosition;
	coord = coord/coord.w;
	if (any(lessThan(coord.st, vec2(0.0))) || any(greaterThan(coord.st, vec2(1.0)))) {
		return;
	}	
	vec2 poissonDisk[4] = vec2[](
	  vec2( -0.94201624, -0.39906216 ),
	  vec2( 0.94558609, -0.76890725 ),
	  vec2( -0.094184101, -0.92938870 ),
	  vec2( 0.34495938, 0.29387760 )
	);

	float visibility = 1.0;
	float depth = shadowPosition.z / shadowPosition.w;
	for (int i = 0; i < 4; ++i) {
		float result = texture2D(shadowMap, vec2(shadowPosition.xy + poissonDisk[i] / 50) / shadowPosition.w).r;
		if (result < depth) {
			visibility -= 0.15;
		}
	}
	color.rgb *= clamp(visibility, 0.0, 1.0);
	glow = vec4(vec3(0), 1.0);

	/*
	if (shadowPosition.w > 0 && distanceFromLight < coord.z) {
		color.rgb = color.rgb * 0.5;
	}

	*/
}
